
/* This program blinks a led without the use of the delay function
 * Multitasking framework used is from paper by Loe Feijs entitled Multi-tasking and Arduino: Why and How?
 * Published in Chen, L.-L., T. Djajadiningrat, L. Feijs, S. Fraser, J. Hu, S. Kyffin and D. Steffen, Eds. (2013). 
 * Design and semantics of form and movement. 8th International Conference on 
 * Design and Semantics of Form and Movement (DeSForM 2013). ISBN 978-90-386-3462-3,  Wuxi, Philips.
 *
 * Software coding Jan Rouvroye for workshop multitasking Arduino.
 * Last edit: May 23, 2014
 */

#define LED_PIN 11                // pin where led is connected
#define LED_OFF 0            
#define LED_ON 1
#define LED_OFF_TIME 250          // number of intervals of 10 ms when the led is off
#define LED_ON_TIME 10            // number of intervals of 10 ms when the led is on

int ledState;                     // variable indicating the state of the led, either LED_OFF or LED_ON
int ledTimer;                     // counter for the number of intervals of 10 ms when timing is needed

void setup() {
  pinMode(LED_PIN, OUTPUT);

  ledState = LED_OFF;             // initial state is LED_ON
  ledTimer = LED_OFF_TIME;
}

void loop() {
  ledStep();

  delay(10);                      // loop once every 10 ms (if no other processes delay execution)
}

void ledStep() {
  switch(ledState) {
  case LED_ON:
    digitalWrite(LED_PIN, HIGH);
    if (ledTimer > 0)            // stay in LED_ON state
      ledTimer = ledTimer - 1;
    else                         // change to LED_OFF state
    {
      ledState = LED_OFF;
      ledTimer = LED_OFF_TIME;   // initialize timer for LED_OFF state
    }
    break;

  case LED_OFF:
    digitalWrite(LED_PIN, LOW);
    if (ledTimer > 0)           // stay in LED_OFF state
      ledTimer = ledTimer - 1;
    else                        // change to LED_ON state
    {
      ledState = LED_ON;
      ledTimer = LED_ON_TIME;   // initialize timer for LED_ON state
    }
    break;
  }
}



